Summary: Library that manage an application's display on terminals
Name: ncurses
Version: 6.4
Release: 4%{?dist}
License: MIT
URL: https://invisible-island.net/%{name}/%{name}.html
Source0: https://github.com/mirror/ncurses/archive/refs/tags/v%{version}.tar.gz
Source1: base.list

# patch to 20230408 for CVE-2023-29491
Patch0001: 20230107.patch
Patch0002: 20230114.patch
Patch0003: 20230121.patch
Patch0004: 20230128.patch
Patch0005: 20230211.patch
Patch0006: 20230218.patch
Patch0007: 20230225.patch
Patch0008: 20230311.patch
Patch0009: 20230401.patch
Patch0010: 20230408.patch

Patch3000: ncurses-config.patch
Patch3001: ncurses-libs.patch 
Patch3002: ncurses-urxvt.patch
Patch3003: ncurses-kbs.patch

BuildRequires: gcc gcc-c++ make pkgconfig 
BuildRequires: gpm-devel git
Requires: %{name}-libs = %{version}-%{release}

%description
The ncurses (new curses) library is a free software emulation of curses in 
System V Release 4.0 (SVr4), and more. It uses terminfo format, supports pads
and color and multiple highlights and forms characters and function-key mapping,
and has all the other SVr4-curses enhancements over BSD curses. SVr4 curses 
became the basis of X/Open Curses.
The distribution includes the library and support utilities, including:
captoinfo, a termcap conversion tool
clear, utility for clearing the screen
infocmp, the terminfo decompiler
tabs, set tabs on a terminal
tic, the terminfo compiler
toe, list (table of) terminfo entries
tput, utility for retrieving terminal capabilities in shell scripts
tset, to initialize the terminal


%package libs
Summary: Ncurses libraries
Requires: %{name}-base = %{version}-%{release}

%description libs
This package contains the ABI version 6 libraries of the ncurses libraries.


%package compat-libs
Summary: Ncurses compatibility libraries
Requires: %{name}-base = %{version}-%{release}

%description compat-libs
This package contains the ABI version 5 of the ncurses libraries for
compatibility.


%package base
Summary: Descriptions of common terminals
BuildArch: noarch

%description base
This package contains descriptions of common terminals. If this package
doesn't provide the terminal you need, please find it in the ncurses-term
package.


%package term
Summary: Terminal descriptions
Requires: %{name}-base = %{version}-%{release}
BuildArch: noarch

%description term
This package contains other all terminal descriptions except those in
the ncurses-base package.


%package devel
Summary: Development files for the ncurses library
Requires: %{name}-libs = %{version}-%{release}
Requires: pkgconfig

%description devel
The header files and libraries for developing applications that use
the ncurses terminal handling library.
Install the ncurses-devel package if you want to develop applications
which will use ncurses.


%package static
Summary: Static libraries for the ncurses library
Requires: %{name}-devel = %{version}-%{release}

%description static
The ncurses-static package includes static libraries of the ncurses library.


%prep
%autosetup -n %{name}-%{version} -p1 -S git


%build
common_options="\
    --enable-colorfgbg \
    --enable-hard-tabs \
    --enable-overwrite \
    --enable-pc-files \
    --enable-xmc-glitch \
    --disable-root-access \
    --disable-root-environ \
    --disable-stripping \
    --disable-wattr-macros \
    --with-cxx-shared \
    --with-ospeed=unsigned \
    --with-pkg-config-libdir=%{_libdir}/pkgconfig \
    --with-shared \
    --with-terminfo-dirs=%{_sysconfdir}/terminfo:%{_datadir}/terminfo \
    --with-termlib=tinfo \
    --with-ticlib=tic \
    --with-xterm-kbs=DEL \
    --without-ada"

# build four different type of libraries 
for targ in narrowc5 narrowc6 widec5 widec6 ; do
   mkdir $targ
   pushd $targ
   ln -s ../configure configure
   case $targ in 
        narrowc5)
           %configure $common_options --with-abi-version=5 --with-chtype=long \
                      --without-progs
           ;;
        widec5)
           %configure $common_options --with-abi-version=5 --with-chtype=long \
                      --without-progs  --enable-widec
           ;;
        narrowc6)
           %configure $common_options --with-abi-version=6 --without-progs
           ;;
        widec6)
           %configure $common_options --with-abi-version=6 --enable-widec
           ;;
        *)
           echo "wrong"
           exit 1
   esac
   %make_build libs
   if [ "$targ" == "widec6" ] ; then
      %make_build -C progs
   fi
   popd
done


%install
make DESTDIR=%{buildroot}/ -C narrowc5  install.libs
make DESTDIR=%{buildroot}/ -C widec5  install.libs
make DESTDIR=%{buildroot}/ -C narrowc6  install.libs
rm %{buildroot}%{_libdir}/lib{tic,tinfo}.so.6*
make DESTDIR=%{buildroot}/ -C widec6 install.{libs,progs,data,includes,man}

chmod 755 %{buildroot}%{_libdir}/lib*.so.*.*
chmod 644 %{buildroot}%{_libdir}/lib*.a

mkdir -p %{buildroot}%{_sysconfdir}/terminfo

# prepare base.list and term.list for terminfo package
cat %{SOURCE1} | sort -u > base.list
find %{buildroot}%{_datadir}/terminfo ! -type d | sed "s|^%{buildroot}||" | \
while read line
do
    echo "%dir $(dirname $line)"
    echo $line
done | sort -u > whole.list
comm -3 whole.list base.list > term.list

# symlink all headers
mkdir %{buildroot}%{_includedir}/ncurses{,w}
for l in %{buildroot}%{_includedir}/*.h; do
    ln -s ../$(basename $l) %{buildroot}%{_includedir}/ncurses
    ln -s ../$(basename $l) %{buildroot}%{_includedir}/ncursesw
done

# ensure old usages like '-lcurses' still can be builded
for l in %{buildroot}%{_libdir}/libncurses{,w}.so; do
    soname=$(basename $(readlink $l))
    rm -f $l
    echo "INPUT($soname -ltinfo)" > $l
done

rm -f %{buildroot}%{_libdir}/libcurses{,w}.so
echo "INPUT(-lncurses)" > %{buildroot}%{_libdir}/libcurses.so
echo "INPUT(-lncursesw)" > %{buildroot}%{_libdir}/libcursesw.so

echo "INPUT(-ltinfo)" > %{buildroot}%{_libdir}/libtermcap.so

rm -f %{buildroot}%{_bindir}/ncurses*5-config
rm -f %{buildroot}%{_libdir}/terminfo
rm -f %{buildroot}%{_libdir}/pkgconfig/*_g.pc


%post -p /sbin/ldconfig libs


%postun -p /sbin/ldconfig libs


%post -p /sbin/ldconfig compat-libs


%postun -p /sbin/ldconfig compat-libs


%files
%doc ANNOUNCE AUTHORS README TO-DO
%{_mandir}/man1/[cirt]*.gz
%{_mandir}/man{5,7}/*
%{_bindir}/[cirt]*


%files libs
%{_libdir}/lib*.so.6*


%files compat-libs
%{_libdir}/lib*.so.5*


%files base -f base.list
%license COPYING
%doc README
%{_datadir}/tabset
%dir %{_sysconfdir}/terminfo
%dir %{_datadir}/terminfo


%files term  -f term.list


%files devel
%doc c++/README*
%doc misc/ncurses.supp
%doc doc/html/hackguide.html
%doc doc/html/ncurses-intro.html
%{_mandir}/man1/ncurses{,w}6-config*
%{_mandir}/man3/*
%{_bindir}/ncurses*-config
%{_libdir}/lib*.so
%{_libdir}/pkgconfig/*.pc
%dir %{_includedir}/ncurses
%dir %{_includedir}/ncursesw
%{_includedir}/ncurses{,w}/*.h
%{_includedir}/*.h


%files static
%{_libdir}/lib*.a


%changelog
* Fri Aug 16 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 6.4-4
- Rebuilt for loongarch release

* Tue Sep 19 2023 jackeyji <jackeyji@tencent.com> - 6.4-3
- fix CVE-2023-29491

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 6.4-2
- Rebuilt for OpenCloudOS Stream 23.09

* Wed Jul 26 2023 Shuo Wang <abushwang@tencent.com> - 6.4-1
- update to 6.4

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 6.3.20220521-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 6.3.20220521-2
- Rebuilt for OpenCloudOS Stream 23

* Tue May 31 2022 doriscchao <doriscchao@tencent.com> - 6.3.20220521-1
- initial build, Version 6.3.20220521, Release 1

